# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# ======================================================================

__version__= '1.1'

ProductName = 'Honeyd Control Center'
ProductFolder = 'HoneydCenter'




from Globals import HTML, HTMLFile, DTMLFile, MessageDialog, Persistent
from ImageFile import ImageFile  
import OFS,Acquisition, AccessControl.Role
import Globals, sys, traceback
from OFS.DTMLDocument import DTMLDocument
from AccessControl import getSecurityManager,ClassSecurityInfo


from OFS.Document import Document
from OFS.Folder import Folder


import os,stat,time,string,whrandom,Products.PythonScripts
from Products.ZSQLMethods.SQL import SQL

from OFS.History import Historical, html_diff
from OFS.SimpleItem import SimpleItem
from OFS.Cache import Cacheable





class zProductDocument(Folder):
    """Zope Product object class"""    
       
    meta_type = ProductName
    icon = 'misc_/%s/icon' % (ProductFolder)

    #manage tabs
    manage_options=(
	{'label':'Contents','action':'mymanage_main','help':('OFSP','ObjectManager_Contents.stx')},
	{'label':'View','action':''},
	{'label':'Options','action':'manage_opt','help':(ProductFolder,'options.stx')},
	{'label':'Systems','action':'manage_syst','help':(ProductFolder,'options.stx')},
	{'label':'Properties','action':'manage_propertiesForm','help':('OFSP','Properties.stx')},
	{'label':'Security','action':'manage_access','help':('OFSP','Security.stx')},
	{'label':'Undo','action':'manage_UndoForm','help':('OFSP','Undo.stx')},
	{'label':'Ownership','action':'manage_owner','help':('OFSP','Ownership.stx')},
        
    )


    all_meta_types=(
    {'name': 'Folder','action' :'manage_addProduct/OFSP/folderAdd'},
    {'name': 'DTML Document','action':'manage_addProduct/OFSP/documentAdd'},
    {'name': 'DTML Method','action'  :'manage_addProduct/OFSP/methodAdd'},
    {'name': 'SQL Reporter','action' :'manage_addProduct/SQLFormsReporter/add_form'},
    {'name': 'User Folder','action': 'manage_addProduct/OFSP/manage_addUserFolder'},
    )

 
    mymanage_main = HTMLFile('dtml/main', globals())
    manage_opt = HTMLFile('dtml/options', globals())
    manage_syst = HTMLFile('dtml/systems', globals())
    OS_List={
	 1:"Microsoft NT 4.0 SP5-SP6",
	 2:"Windows NT4 / Win95 / Win98",
	 3:"Windows NT 4.0 Server SP5-SP6",
	 4:"MS Windows2000 Professional RC1/W2K Advance Server Beta3",
	 5:"Windows NT 4.0 SP 6a + hotfixes",
	 6:"Microsoft Windows.NET Enterprise Server (build 3615 beta)",
	 7:"Windows Millennim Edition (Me), Win 2000, or WinXP",

	20:"OpenBSD 2.9-stable",
	21:"OpenBSD 3.0 (x86 or SPARC)",

	30:"Linux 2.2.5 - 2.2.13 SMP",
	31:"Linux 2.3.49 x86",
	32:"Linux Kernel 2.4.3 SMP (RedHat)",
	33:"Linux Kernel 2.4.0 - 2.4.18 (X86)",

	40:"HP-UX 11.00",
}

    def __init__(self, id, title,database,urlupdate):
        """ init method """
	self.inited=0
        self.ProductFolder = ProductFolder
        self.id = id
        self.title = title
	self.database = database
	self.sq_table = 'honey_'+str(id)
	self.honeydpid = '/var/run/honeyd.pid'
	self.arpdpid = '/var/run/arpd.pid'
	self.workdir = '/tmp/honeyd/'
	self.user = self.mk_rand_string(15)
	self.password = self.mk_rand_string(15)
	self.urlupdate = urlupdate
	self.items = [
	{'os':7,'ports':'135,139,445','ip':'default','uptime':'none','uid':'-','gid':'-'},
	{'os':21,'ports':'13,22,37,113','ip':'10.0.0.143','uptime':'3','uid':'32767','gid':'32767'},
	{'os':20,'ports':'22,80','ip':'10.0.0.157','uptime':'15','uid':'32767','gid':'32767'},
	]

	#set security
	self.manage_addUserFolder()
	self.acl_users.userFolderAddUser(name=self.user,password=self.password,roles=('Manager'),domains=())
	#self.acl_users.userFolderDelUsers(('dan',))
	
	

	#add some objects
	self.manage_addDTMLMethod(id='index_html', title='this is the index')
	self.index_html.raw="""<dtml-var standard_html_header>
<h1><dtml-var title_or_id></h1>
<h2>IDS Reports</h2>
<ul>
<dtml-in "objectItems(['SQL Form Reporter'])">
<li><a href="<dtml-var id>/input"><dtml-var title_or_id></a></li>
</dtml-in>
</ul>
<dtml-var standard_html_footer>
"""

	self.manage_addDTMLMethod(id='denied', title='Access denied')
	self.denied.raw="<h1>Access denied!</h1>"

	self.manage_addDTMLDocument(id='README',title='Readme first')
	self.README.raw="""
Honeyd control center
Add your own SQL Form Reporters
"""
	self.rebuild_honeyd_conf()

    def mk_rand_string(self,n):
	chars=string.digits+' '+string.letters
	s=''
	for i in range(n):
	    s=s+chars[whrandom.randint(0,len(chars)-1)]

	return s


    def manage_opt_action(self, title,database,
	honeydpid,arpdpid,workdir,REQUEST=None):
        """ update properties  """
        self.title = title
	self.database = database
	self.honeydpid = honeydpid
	self.arpdpid = arpdpid
	self.workdir = workdir
	#erase previous user
	try: self.acl_users.userFolderDelUsers((self.user,))
	except: pass
	#generate new user
	self.user = self.mk_rand_string(15)
	self.password = self.mk_rand_string(15)
	#add new user
	self.acl_users.userFolderAddUser(self.user,self.password,('Manager',),'')
	#self.acl_users.userFolderEditUser(self.user,'123',('Manager',),'')
	if self.workdir[-1] != '/': self.workdir=self.workdir+'/'

	if self.inited == 0:
	    self.manage_permission('View',roles=['Manager'],acquire=0)
	    self.manage_permission('View management screens',roles=['Manager'],acquire=0)

	    self.manage_addProduct['PythonScripts'].manage_addPythonScript(id='valid_admin')
	    self.valid_admin.ZPythonScript_setTitle('Only Zope Admin')
	    self.valid_admin.ZPythonScript_edit('','''#
#This py script check if current user is Zope manager
#
if context.REQUEST.AUTHENTICATED_USER.has_role('Manager'): return 1
else: return 0
''')
	    self.manage_addProduct['PythonScripts'].manage_addPythonScript(id='updatedb')
	    self.updatedb.ZPythonScript_setTitle('Insert data values')
	    self.updatedb.ZPythonScript_edit('','''#
# insert SQL data values
#
REQUEST=context.REQUEST
SQL=context.form.SQL

datetime = int(REQUEST.datetime)
port = str(REQUEST.port)
src_ip = str(REQUEST.src_ip)
src_host = str(REQUEST.src_host)
dst_ip = str(REQUEST.dst_ip)
dst_host=  str(REQUEST.dst_host)
extra = str(REQUEST.extra)
q="insert into %s(datetime,port,src_ip,src_host,dst_ip,dst_host,extra) values(%%d,'%%s','%%s','%%s','%%s','%%s','%%s')" %% (datetime,port,src_ip,src_host,dst_ip,dst_host,extra)
#print q
SQL(q=q)
print "Ok! Data inserted" #nobody will see this message!

return printed
''' % (self.sq_table))

	    self.manage_addProduct['SQLForms'].manage_addSQLForms(id='form',title='',sq_table='-',database=self.database)
	    self.form.set_fields(
	[
	{'fieldlabel': 'key', 'options': 'inc', 'unique': 'pri', 'expr': 1, 'form_type': 'text', 'form_size': 1, 'fieldname': 'autoid', 'sq_size': 1, 'sq_type': 'integer', 'mandatory': 0},
	{'fieldlabel': 'datetime', 'options': 'st', 'unique': 'none', 'expr': '', 'form_type': 'date', 'form_size': 10, 'fieldname': 'datetime', 'sq_size': 1, 'sq_type': 'integer', 'mandatory': 0},
	{'fieldlabel': 'port', 'options': 'dict', 'unique': 'none', 'expr': "{-1:'-select port-',13:'daytime',20:'ftp data',21:'ftp',22:'ssh',23:'telnet',80:'www',109:'pop2',110:'pop3',137:'netbios-ns',138:'netbios-dgm',139:'netbios-ssn',143:'imap',161:'snmp',220:'imap3',389:'ldap',443:'https'}", 'form_type': 'select', 'form_size': 10, 'fieldname': 'port', 'sq_size': 5, 'sq_type': 'varchar', 'mandatory': 0},
	{'fieldlabel': 'src_ip', 'options': 'st', 'unique': 'none', 'expr': '', 'form_type': 'text', 'form_size': 10, 'fieldname': 'src_ip', 'sq_size': 25, 'sq_type': 'varchar', 'mandatory': 0},
	{'fieldlabel': 'src_host', 'options': 'st', 'unique': 'none', 'expr': '', 'form_type': 'text', 'form_size': 10, 'fieldname': 'src_host', 'sq_size': 128, 'sq_type': 'varchar', 'mandatory': 0},
	{'fieldlabel': 'dst_ip', 'options': 'st', 'unique': 'none', 'expr': '', 'form_type': 'text', 'form_size': 10, 'fieldname': 'dst_ip', 'sq_size': 25, 'sq_type': 'varchar', 'mandatory': 0},
	{'fieldlabel': 'dst_host', 'options': 'st', 'unique': 'none', 'expr': '', 'form_type': 'text', 'form_size': 10, 'fieldname': 'dst_host', 'sq_size': 128, 'sq_type': 'varchar', 'mandatory': 0},
	{'fieldlabel': 'extra', 'options': 'st', 'unique': 'none', 'expr': '', 'form_type': 'textarea', 'form_size': 60, 'fieldname': 'extra', 'sq_size': 250, 'sq_type': 'varchar', 'mandatory': 0}
	])
	    self.form.manage_opt_action(title='SQL Form Object',
		database=self.database,sq_table=self.sq_table,
		security_add='valid_admin',security_list='valid_admin',
	        security_edit='valid_admin',denied_method='denied',
		hide_key='no',headers='yes',
		viewlistlen=100,internal_title_color='#eeeeee',
		internal_c1_color='#ffeeee',internal_c2_color='#eeeeff',
		date_format='%Y/%m/%d-%H:%M:%S',table_cell_spacing=3,
		textarea_rows=3,sql_server_type=4)
	    self.form.manage_sql(act='Create table')

	    #add report #1
	    self.manage_addProduct['SQLFormsReporter'].manage_add(id='r1',title='',database=self.database)
	    self.r1.input_fields=[
    {'label': 'select port', 'cond_type': '=', 'options': "{-1:'-select port-',13:'daytime',20:'ftp data',21:'ftp',22:'ssh',23:'telnet',80:'www',109:'pop2',110:'pop3',137:'netbios-ns',138:'netbios-dgm',139:'netbios-ssn',143:'imap',161:'snmp',220:'imap3',389:'ldap',443:'https'}", 's_table': 'form', 's_field': 'port', 'opt_or': 0, 'f_type': 'select', 'opt_type': 'dict'},
    {'label': 'or input port', 'cond_type': '=', 'options': '', 's_table': 'form', 's_field': 'port', 'opt_or': 1, 'f_type': 'text', 'opt_type': 'dict'}
    ]
	    self.r1.select_fields=[
    {'label': 'datetime', 's_field': 'datetime', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'date', 'as': 'expr5', 'visible': 1, 'j_table': '0'},
    {'label': 'port numeric', 's_field': 'port', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'text', 'as': 'expr2', 'visible': 1, 'j_table': '0'},
    {'label': 'port name', 's_field': 'port', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'select', 'as': 'expr3', 'visible': 1, 'j_table': '0'},
    {'label': 'src_ip', 's_field': 'src_ip', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'text', 'as': 'expr1', 'visible': 1, 'j_table': '0'},
    {'label': 'src_host', 's_field': 'src_host', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'text', 'as': 'expr4', 'visible': 1, 'j_table': '0'},
    {'label': 'dst_ip', 's_field': 'dst_ip', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'text', 'as': 'expr6', 'visible': 1, 'j_table': '0'},
    {'label': 'dst_host', 's_field': 'dst_host', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'text', 'as': 'expr7', 'visible': 1, 'j_table': '0'},
    {'label': 'extra', 's_field': 'extra', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'textarea', 'as': 'expr8', 'visible': 1, 'j_table': '0'}
    ]
	    self.r1.manage_opt_action(title='List by port',
	        database=self.database,
		security_input='valid_admin',
		security_report='valid_admin',
		denied_method='denied')
	    
	    #add report #2
	    self.manage_addProduct['SQLFormsReporter'].manage_add(id='r2',title='',database=self.database)
	    self.r2.input_fields=[
    {'label': 'start date', 'cond_type': '>', 'options': '', 's_table': 'form', 's_field': 'datetime', 'opt_or': 0, 'f_type': 'date', 'opt_type': 'st'},
    ]
	    self.r2.select_fields=[
    {'label': 'datetime', 's_field': 'datetime', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'date', 'as': 'expr5', 'visible': 1, 'j_table': '0'},
    {'label': 'port numeric', 's_field': 'port', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'text', 'as': 'expr2', 'visible': 1, 'j_table': '0'},
    {'label': 'port name', 's_field': 'port', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'select', 'as': 'expr3', 'visible': 1, 'j_table': '0'},
    {'label': 'src_ip', 's_field': 'src_ip', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'text', 'as': 'expr1', 'visible': 1, 'j_table': '0'},
    {'label': 'src_host', 's_field': 'src_host', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'text', 'as': 'expr4', 'visible': 1, 'j_table': '0'},
    {'label': 'dst_ip', 's_field': 'dst_ip', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'text', 'as': 'expr6', 'visible': 1, 'j_table': '0'},
    {'label': 'dst_host', 's_field': 'dst_host', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'text', 'as': 'expr7', 'visible': 1, 'j_table': '0'},
    {'label': 'extra', 's_field': 'extra', 'j_field': 'Nothing', 's_table': 'form', 'f_type': 'textarea', 'as': 'expr8', 'visible': 1, 'j_table': '0'}
    ]
	    self.r2.manage_opt_action(title='List latest events',
	        database=self.database,
		security_input='valid_admin',
		security_report='valid_admin',
		denied_method='denied')
	    #print "inited"
	    self.inited=1
	self.rebuild_honeyd_conf()
	if REQUEST is not None:
            return Globals.MessageDialog( title='Edited', message='<strong>%s</strong> has been edited.' % self.id, action ='manage_opt',)

    def xx(self):
	""" sss """
	return str(self.r2.input_fields)
	
    def manage_syst_action(self,act='',ix=[],ports=[],ip=[],uptime=[],os=[],uid=[],gid=[],REQUEST=None):
        """ update system list  """
	if act == 'Delete records':
	    L=[]
	    for i in range(len(self.items)):
	      if i not in ix: L.append(self.items[i])
	    self.items=L
	if act == 'Add/Update Systems':
	    L=[]
	    for i in range(len(os)):
	      if ip[i]!='new ip':
	        L.append({'os':os[i],'ports':ports[i],'ip':ip[i],'uptime':uptime[i],'uid':uid[i],'gid':gid[i]})
	    self.items=L
	self.rebuild_honeyd_conf()
	
	if REQUEST is not None:
            return Globals.MessageDialog( title='Edited', message='<strong>%s</strong> has been edited.' % self.id, action ='manage_syst',)


    def rebuild_honeyd_conf(self):
	try: os.makedirs(self.workdir)
	except: pass
	os.chmod(self.workdir,0777)
	f=open(self.workdir+'timeoutsocket.py','wt')
	os.chmod(self.workdir+'timeoutsocket.py',0666)
	f.write(timeoutsocket)
	f.close()
	f=open(self.workdir+'start-honeyd','wt')
	f.write('''#!/bin/sh
honeyd -f %shoneyd.conf #-l %shoneyd.log
''' % (self.workdir,self.workdir))
	f.close()
	os.chmod(self.workdir+'start-honeyd',0777)
	f=open(self.workdir+'stop-honeyd','wt')
	f.write('#!/bin/sh\nkill -9 `cat %s`\n' % (self.honeydpid))
	f.close()
	os.chmod(self.workdir+'stop-honeyd',0777)
	f=open(self.workdir+'fakescript.py','wt')
	f.write('''#python fake script

import os,time,socket,sys
import httplib,urllib,string,base64

sys.path.append('%s')
import timeoutsocket
timeoutsocket.setDefaultSocketTimeout(60)


def get_hostname(ip):
   try:
      host, aliaslist, ipaddrlist = socket.gethostbyaddr(ip)
   except:
     try:
      (stdin,stdout)=os.popen2('nbtscan -q '+ip)
      stdin.close()
      s=stdout.read()
      stdout.close()
      L=[]
      for i in s.split(" "):
          if i!='': L.append(i)
      if L[1]==L[3]: host=L[1]
      else: host=L[1]+":"+L[3]
     except:
      host="-"
   return host
     
     
      

#get required info
IP_SRC = sys.argv[1]
IP_DST = sys.argv[2]
DST_PORT = sys.argv[3]

#now=time.strftime('%%A, %%d %%B, %%Y - %%M:%%S%%p',time.localtime())
now=int(time.mktime(time.localtime()))
srchost = get_hostname(IP_SRC)
dsthost = get_hostname(IP_DST)


params=urllib.urlencode({'datetime':now,'port':DST_PORT,
    'src_ip':IP_SRC,'src_host':srchost,
    'dst_ip':IP_DST,'dst_host':dsthost,'extra':'-'})

auth = string.strip(base64.encodestring('%s:%s'))
#to decode:  base64.decodestring(auth)

headers={"Content-type":"application/x-www-form-urlencoded",
    "Accept":"text/plain","Authorization":"Basic "+auth}
conn=httplib.HTTPConnection("localhost")
conn.request("POST","%s",params,headers)
response=conn.getresponse()
data=response.read()
if response.status != 200:
    print response.status,response.reason
    print data
conn.close()


#display a generic message
print "Connection to host lost."
''' % (self.workdir,self.user,self.password,self.urlupdate))
	f.close()
	os.chmod(self.workdir+'fakescript.py',0666)
	s='''#honeyd.conf - honeyd config file
###########################################
#This file is automatically generated     #
#by Honeyd Control Center (Zope Version)  #
#(C) 2003 Ioan Coman - ioan2k@yahoo.com   #
###########################################

'''
	all_ips=0
	ip_list=[]
	an_list=[] #annotate index list
	for i in self.items:
	  if i['os'] not in an_list: an_list.append(i['os'])
	for i in an_list:
	  s=s+'annotate "%s" fragment old\n' % (self.OS_List[i])
	ix=0
	for i in self.items:
	  template='template'+str(ix)
	  if i['ip'] == 'default':
		template='default'
		all_ips=1
	  else: ip_list.append(i['ip'])
	  s=s+'''
create %s
set %s personality "%s"
''' % (template,template,self.OS_List[i['os']],)
	  for p in i['ports'].split(","):
	    s=s+'add %s tcp port %s "python %sfakescript.py $ipsrc $ipdst $dport"\n' % (template,str(p),self.workdir)
	  s=s+'set %s default tcp action reset\n' % (template)
	  if template!='default':
	    s=s+'set %s uid %s gid %s\n' % (template,str(i['uid']),str(i['gid']))
	    s=s+'bind %s %s\n' % (str(i['ip']),template)
	    s=s+'set %s uptime %d\n' % (str(i['ip']),int(i['uptime'])*3600*24)
	    ix=ix+1
	s=s+'\n\n'
	self.honeyd_conf = Document(s, __name__='honeyd_conf')
	#save honeyd.conf
	f=open(self.workdir+'honeyd.conf','wt')
	f.write(s)
	f.close()
	#start arpd
	if all_ips==1: XX=''
	else: XX=' '+string.join(ip_list,' ')
	f=open(self.workdir+'start-arpd','wt')
	f.write('#!/bin/sh\narpd%s\n' % (XX))
	f.close()
	os.chmod(self.workdir+'start-arpd',0777)
	#stop arpd
	f=open(self.workdir+'stop-arpd','wt')
	f.write('#!/bin/sh\nkill -9 `cat %s`\n' % (self.arpdpid))
	f.close()
	os.chmod(self.workdir+'stop-arpd',0777)



Globals.default__class_init__(zProductDocument)


timeoutsocket = '''
####
# Copyright 2000,2001 by Timothy O'Malley <timo@alum.mit.edu>
# 
#                All Rights Reserved
# 
# Permission to use, copy, modify, and distribute this software
# and its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of
# Timothy O'Malley  not be used in advertising or publicity
# pertaining to distribution of the software without specific, written
# prior permission. 
# 
# Timothy O'Malley DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS, IN NO EVENT SHALL Timothy O'Malley BE LIABLE FOR
# ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
####

"""Timeout Socket

This module enables a timeout mechanism on all TCP connections.  It
does this by inserting a shim into the socket module.  After this module
has been imported, all socket creation goes through this shim.  As a
result, every TCP connection will support a timeout.

The beauty of this method is that it immediately and transparently
enables the entire python library to support timeouts on TCP sockets.
As an example, if you wanted to SMTP connections to have a 20 second
timeout:

    import timeoutsocket
    import smtplib
    timeoutsocket.setDefaultSocketTimeout(20)


The timeout applies to the socket functions that normally block on
execution:  read, write, connect, and accept.  If any of these 
operations exceeds the specified timeout, the exception Timeout
will be raised.

The default timeout value is set to None.  As a result, importing
this module does not change the default behavior of a socket.  The
timeout mechanism only activates when the timeout has been set to
a numeric value.  (This behavior mimics the behavior of the
select.select() function.)

This module implements two classes: TimeoutSocket and TimeoutFile.

The TimeoutSocket class defines a socket-like object that attempts to
avoid the condition where a socket may block indefinitely.  The
TimeoutSocket class raises a Timeout exception whenever the
current operation delays too long. 

The TimeoutFile class defines a file-like object that uses the TimeoutSocket
class.  When the makefile() method of TimeoutSocket is called, it returns
an instance of a TimeoutFile.

Each of these objects adds two methods to manage the timeout value:

    get_timeout()   -->  returns the timeout of the socket or file
    set_timeout()   -->  sets the timeout of the socket or file


As an example, one might use the timeout feature to create httplib
connections that will timeout after 30 seconds:

    import timeoutsocket
    import httplib
    H = httplib.HTTP("www.python.org")
    H.sock.set_timeout(30)

Note:  When used in this manner, the connect() routine may still
block because it happens before the timeout is set.  To avoid
this, use the 'timeoutsocket.setDefaultSocketTimeout()' function.

Good Luck!

"""

__version__ = "$Revision: 1.23 $"
__author__  = "Timothy O'Malley <timo@alum.mit.edu>"

#
# Imports
#
import select, string
import socket
if not hasattr(socket, "_no_timeoutsocket"):
    _socket = socket.socket
else:
    _socket = socket._no_timeoutsocket


#
# Set up constants to test for Connected and Blocking operations.
# We delete 'os' and 'errno' to keep our namespace clean(er).
# Thanks to Alex Martelli and G. Li for the Windows error codes.
#
import os
if os.name == "nt":
    _IsConnected = ( 10022, 10056 )
    _ConnectBusy = ( 10035, )
    _AcceptBusy  = ( 10035, )
else:
    import errno
    _IsConnected = ( errno.EISCONN, )
    _ConnectBusy = ( errno.EINPROGRESS, errno.EALREADY, errno.EWOULDBLOCK )
    _AcceptBusy  = ( errno.EAGAIN, errno.EWOULDBLOCK )
    del errno
del os


#
# Default timeout value for ALL TimeoutSockets
#
_DefaultTimeout = None
def setDefaultSocketTimeout(timeout):
    global _DefaultTimeout
    _DefaultTimeout = timeout
def getDefaultSocketTimeout():
    return _DefaultTimeout

#
# Exceptions for socket errors and timeouts
#
Error = socket.error
class Timeout(Exception):
    pass


#
# Factory function
#
from socket import AF_INET, SOCK_STREAM
def timeoutsocket(family=AF_INET, type=SOCK_STREAM, proto=None):
    if family != AF_INET or type != SOCK_STREAM:
        if proto:
            return _socket(family, type, proto)
        else:
            return _socket(family, type)
    return TimeoutSocket( _socket(family, type), _DefaultTimeout )
# end timeoutsocket

#
# The TimeoutSocket class definition
#
class TimeoutSocket:
    """TimeoutSocket object
    Implements a socket-like object that raises Timeout whenever
    an operation takes too long.
    The definition of 'too long' can be changed using the
    set_timeout() method.
    """

    _copies = 0
    _blocking = 1
    
    def __init__(self, sock, timeout):
        self._sock     = sock
        self._timeout  = timeout
    # end __init__

    def __getattr__(self, key):
        return getattr(self._sock, key)
    # end __getattr__

    def get_timeout(self):
        return self._timeout
    # end set_timeout

    def set_timeout(self, timeout=None):
        self._timeout = timeout
    # end set_timeout

    def setblocking(self, blocking):
        self._blocking = blocking
        return self._sock.setblocking(blocking)
    # end set_timeout

    def connect_ex(self, addr):
        errcode = 0
        try:
            self.connect(addr)
        except Error, why:
            errcode = why[0]
        return errcode
    # end connect_ex
        
    def connect(self, addr, port=None, dumbhack=None):
        # In case we were called as connect(host, port)
        if port != None:  addr = (addr, port)

        # Shortcuts
        sock    = self._sock
        timeout = self._timeout
        blocking = self._blocking

        # First, make a non-blocking call to connect
        try:
            sock.setblocking(0)
            sock.connect(addr)
            sock.setblocking(blocking)
            return
        except Error, why:
            # Set the socket's blocking mode back
            sock.setblocking(blocking)
            
            # If we are not blocking, re-raise
            if not blocking:
                raise
            
            # If we are already connected, then return success.
            # If we got a genuine error, re-raise it.
            errcode = why[0]
            if dumbhack and errcode in _IsConnected:
                return
            elif errcode not in _ConnectBusy:
                raise
            
        # Now, wait for the connect to happen
        # ONLY if dumbhack indicates this is pass number one.
        #   If select raises an error, we pass it on.
        #   Is this the right behavior?
        if not dumbhack:
            r,w,e = select.select([], [sock], [], timeout)
            if w:
                return self.connect(addr, dumbhack=1)

        # If we get here, then we should raise Timeout
        raise Timeout("Attempted connect to %s timed out." % str(addr) )
    # end connect

    def accept(self, dumbhack=None):
        # Shortcuts
        sock     = self._sock
        timeout  = self._timeout
        blocking = self._blocking

        # First, make a non-blocking call to accept
        #  If we get a valid result, then convert the
        #  accept'ed socket into a TimeoutSocket.
        # Be carefult about the blocking mode of ourselves.
        try:
            sock.setblocking(0)
            newsock, addr = sock.accept()
            sock.setblocking(blocking)
            timeoutnewsock = self.__class__(newsock, timeout)
            timeoutnewsock.setblocking(blocking)
            return (timeoutnewsock, addr)
        except Error, why:
            # Set the socket's blocking mode back
            sock.setblocking(blocking)

            # If we are not supposed to block, then re-raise
            if not blocking:
                raise
            
            # If we got a genuine error, re-raise it.
            errcode = why[0]
            if errcode not in _AcceptBusy:
                raise
            
        # Now, wait for the accept to happen
        # ONLY if dumbhack indicates this is pass number one.
        #   If select raises an error, we pass it on.
        #   Is this the right behavior?
        if not dumbhack:
            r,w,e = select.select([sock], [], [], timeout)
            if r:
                return self.accept(dumbhack=1)

        # If we get here, then we should raise Timeout
        raise Timeout("Attempted accept timed out.")
    # end accept

    def send(self, data, flags=0):
        sock = self._sock
        if self._blocking:
            r,w,e = select.select([],[sock],[], self._timeout)
            if not w:
                raise Timeout("Send timed out")
        return sock.send(data, flags)
    # end send

    def recv(self, bufsize, flags=0):
        sock = self._sock
        if self._blocking:
            r,w,e = select.select([sock], [], [], self._timeout)
            if not r:
                raise Timeout("Recv timed out")
        return sock.recv(bufsize, flags)
    # end recv

    def makefile(self, flags="r", bufsize=-1):
        self._copies = self._copies +1
        return TimeoutFile(self, flags, bufsize)
    # end makefile

    def close(self):
        if self._copies <= 0:
            self._sock.close()
        else:
            self._copies = self._copies -1
    # end close

# end TimeoutSocket


class TimeoutFile:
    """TimeoutFile object
    Implements a file-like object on top of TimeoutSocket.
    """
    
    def __init__(self, sock, mode="r", bufsize=4096):
        self._sock          = sock
        self._bufsize       = 4096
        if bufsize > 0: self._bufsize = bufsize
        if not hasattr(sock, "_inqueue"): self._sock._inqueue = ""

    # end __init__

    def __getattr__(self, key):
        return getattr(self._sock, key)
    # end __getattr__

    def close(self):
        self._sock.close()
        self._sock = None
    # end close
    
    def write(self, data):
        self.send(data)
    # end write

    def read(self, size=-1):
        _sock = self._sock
        _bufsize = self._bufsize
        while 1:
            datalen = len(_sock._inqueue)
            if datalen >= size >= 0:
                break
            bufsize = _bufsize
            if size > 0:
                bufsize = min(bufsize, size - datalen )
            buf = self.recv(bufsize)
            if not buf:
                break
            _sock._inqueue = _sock._inqueue + buf
        data = _sock._inqueue
        _sock._inqueue = ""
        if size > 0 and datalen > size:
            _sock._inqueue = data[size:]
            data = data[:size]
        return data
    # end read

    def readline(self, size=-1):
        _sock = self._sock
        _bufsize = self._bufsize
        while 1:
            idx = string.find(_sock._inqueue, "\\n")
            if idx >= 0:
                break
            datalen = len(_sock._inqueue)
            if datalen >= size >= 0:
                break
            bufsize = _bufsize
            if size > 0:
                bufsize = min(bufsize, size - datalen )
            buf = self.recv(bufsize)
            if not buf:
                break
            _sock._inqueue = _sock._inqueue + buf

        data = _sock._inqueue
        _sock._inqueue = ""
        if idx >= 0:
            idx = idx + 1
            _sock._inqueue = data[idx:]
            data = data[:idx]
        elif size > 0 and datalen > size:
            _sock._inqueue = data[size:]
            data = data[:size]
        return data
    # end readline

    def readlines(self, sizehint=-1):
        result = []
        data = self.read()
        while data:
            idx = string.find(data, "\\n")
            if idx >= 0:
                idx = idx + 1
                result.append( data[:idx] )
                data = data[idx:]
            else:
                result.append( data )
                data = ""
        return result
    # end readlines

    def flush(self):  pass

# end TimeoutFile


#
# Silently replace the socket() builtin function with
# our timeoutsocket() definition.
#
if not hasattr(socket, "_no_timeoutsocket"):
    socket._no_timeoutsocket = socket.socket
    socket.socket = timeoutsocket
del socket
socket = timeoutsocket
# Finis
'''
